unit EvalCom;

interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  StdCtrls,Dialogs,EValDefs,Eval;

const HistorySeparator='|';

type

  TEvalCombo = class(TComboBox)
  private
    FPosition : Integer;
    procedure SetPosition(Value:Integer);
  protected
    {procedure MyDrawItem(Control: TWinControl; Index: Integer; Rect: TRect;
              State: TOwnerDrawState);}
      procedure   DropDown;          override;
  public
    constructor Create(AOwner:TComponent); override;
    function GetTextByIndex(_Index,_Pos:Integer):String;
    procedure CreateParams(var Params: TCreateParams);  override;
  published
    property Position:Integer Read FPosition Write SetPosition;
    property ItemIndex;
  end;

  TEvalFnGroupCombo = Class(TComboBox)
  private
        procedure SetFunctionType(Value:TFnType);
        function  GetFunctionType:TFnType;
  protected
        procedure CreateWnd;   override;
  published
        property FunctionType:TFnType Read GetFunctionType Write SetFunctionType;
        property Items Stored False;               
  end;

  TEvalFnGroupList = Class(TListBox)
  private
        procedure SetFunctionType(Value:TFnType);
        function  GetFunctionType:TFnType;
  protected
        procedure CreateWnd;   override;
  published
        property Items Stored False;
        property FunctionType:TFnType Read GetFunctionType Write SetFunctionType;
  end;

  TEvalFnComboBox = class(TComboBox)
  private
        FFnType  : TFnType;
        procedure SetFunctionType(Value:TFnType);
  protected
        procedure CreateWnd;   override;
        procedure LoadFunctions;
  public
        function  GetSelectedFunction:TEvalSymbol;
        function  GetSelectedFunctionHint:String;
        procedure SetSelectedFunction(Value:TEvalSymbol);
  published
        property Items Stored False;
        property SelectedFunction:TEvalSymbol Read GetSelectedFunction
                                              Write SetSelectedFunction;
        property FunctionType:TFnType Read FFnType Write SetFunctionType;
  end;

  TEvalFnListBox = class(TListBox)
  private
        FFnType  : TFnType;
        procedure SetFunctionType(Value:TFnType);
  protected
        procedure CreateWnd;   override;
        procedure LoadFunctions;
  public
        function  GetSelectedFunction:TEvalSymbol;
        function  GetSelectedFunctionHint:String;
        procedure SetSelectedFunction(Value:TEvalSymbol);
  published
        property Items Stored False;
        property SelectedFunction:TEvalSymbol Read GetSelectedFunction
                                              Write SetSelectedFunction;
        property FunctionType:TFnType Read FFnType Write SetFunctionType;
  end;

  TEvalHistoryCombo=class(TEvalCombo)
  private
       FFileName     : String;
       FFileNameKey  : String;
       FNeedSave     : Boolean;
       FEval         : TEval;
       procedure SetFileName(Value:String);
       procedure SetFileNameKey(Value:String);
  protected
       procedure DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
  public
       constructor Create(AOwner:TComponent); override;
       procedure   Notification(AComponent: TComponent; Operation: TOperation); override;
       procedure   Change;                    override;
       procedure   LoadItems;
       procedure   SaveItems;
       procedure   AddWithQuestion(Expr:String);
       procedure   AddCond(Expr:String; Str:String);
       function    Expression:String;
       function    GetExpressionByIndex(idx:Integer):String;
       function    GetCaption:String;
       function    GetCaptionByIndex(idx:Integer):String;
  published
       property Eval:TEval         Read FEval        Write FEval;
       property FileName:String    Read FFileName    Write SetFileName;
       property FileNameKey:String Read FFileNameKey Write SetFileNameKey;
       property Caption:String     Read GetCaption   Stored False;
  end;


implementation
uses EvalRes;


procedure TEvalFnGroupCombo.CreateWnd;
begin
     inherited CreateWnd;
     Style     := csDropDownList;
     GetFunctionGroup(Items);
     ItemIndex := 0;
end;

procedure TEvalFnGroupCombo.SetFunctionType(Value:TFnType);
var i:Integer;
begin
    for i:=0 to Items.Count-1 do
        if Value=TFnType(Items.Objects[i]) then begin
           ItemIndex:=i;
           Exit;
        end;
end;

function  TEvalFnGroupCombo.GetFunctionType:TFnType;
begin
     if ItemIndex>=0 then Result:=TFnType(Items.Objects[ItemIndex])
     else                 Result:=fnNone;
end;



procedure TEvalFnGroupList.CreateWnd;
begin
     inherited CreateWnd;
     GetFunctionGroup(Items);
     ItemIndex := 0;
end;

procedure TEvalFnGroupList.SetFunctionType(Value:TFnType);
var i:Integer;
begin
    for i:=0 to Items.Count-1 do
        if Value=TFnType(Items.Objects[i]) then begin
           ItemIndex:=i;
           Exit;
        end;
end;

function  TEvalFnGroupList.GetFunctionType:TFnType;
begin
     if ItemIndex>=0 then Result:=TFnType(Items.Objects[ItemIndex])
     else                 Result:=fnNone;
end;



// Function ListBox


procedure TEvalFnListBox.CreateWnd;
begin
     inherited CreateWnd;
     FFnType := fnString;
     GetFunctionNames([FFnType],Items);
end;

procedure TEvalFnListBox.SetFunctionType(Value:TFnType);
begin
    if Value<>FFnType then begin
       FFnType:=Value;
       GetFunctionNames([FFnType],Items);
       ItemIndex := 0;
       Click;
    end;
end;

procedure TEvalFnListBox.LoadFunctions;
begin
     Items.Clear;
     GetFunctionNames([FFnType],Items);
end;

function  TEvalFnListBox.GetSelectedFunction:TEvalSymbol;
var F:TDefFun;
begin
     if ItemIndex<0 then Result:=esNone
     else                begin
        F:=GetFunctionByIndex(Integer(Items.Objects[ItemIndex]));
        Result:=F.FT;
     end;
end;

function  TEvalFnListBox.GetSelectedFunctionHint:String;
var F:TDefFun;
begin
     if ItemIndex<0 then Result:=''
     else                begin
        F:=GetFunctionByIndex(Integer(Items.Objects[ItemIndex]));
        Result:=F.Hint;
     end;
end;

procedure TEvalFnListBox.SetSelectedFunction(Value:TEvalSymbol);
var F:TDefFun;
    i,idx:Integer;
begin
     if Value<>SelectedFunction then begin
        F:=GetFunctionBySymbol(Value);
        FunctionType:=F.Kind;
        for i:=0 to Items.Count-1 do begin
            idx:=Integer(Items.Objects[i]);
            F  :=GetFunctionByIndex(idx);
            if F.FT=Value then begin
                 ItemIndex:=i;
                 Exit;
            end;
        end;
     end;
end;


procedure TEvalFnComboBox.CreateWnd;
begin
     inherited CreateWnd;
     FFnType   := fnAll;
     Style     := csDropDownList;
end;

procedure TEvalFnComboBox.SetFunctionType(Value:TFnType);
begin
    if Value<>FFnType then begin
       FFnType:=Value;
       GetFunctionNames([FFnType],Items);
       ItemIndex := 0;
       Click;
    end;
end;

procedure TEvalFnComboBox.LoadFunctions;
begin
     Items.Clear;
end;

function  TEvalFnComboBox.GetSelectedFunction:TEvalSymbol;
var F:TDefFun;
begin
     if ItemIndex<0 then Result:=esNone
     else                begin
        F:=GetFunctionByIndex(Integer(Items.Objects[ItemIndex]));
        Result:=F.FT;
     end;
end;
function  TEvalFnComboBox.GetSelectedFunctionHint:String;
var F:TDefFun;
begin
     if ItemIndex<0 then Result:=''
     else                begin
        F:=GetFunctionByIndex(Integer(Items.Objects[ItemIndex]));
        Result:=F.Hint;
     end;
end;

procedure TEvalFnComboBox.SetSelectedFunction(Value:TEvalSymbol);
var F:TDefFun;
    i,idx:Integer;
begin
     if Value<>SelectedFunction then begin
        F:=GetFunctionBySymbol(Value);
        if (FunctionType<>fnAll) then FunctionType:=F.Kind;
        for i:=0 to Items.Count-1 do begin
            idx:=Integer(Items.Objects[i]);
            F  :=GetFunctionByIndex(idx);
            if F.FT=Value then begin
                 ItemIndex:=i;
                 Exit;
            end;
        end;
     end;
end;





constructor TEvalHistoryCombo.Create(AOwner:TComponent);
begin
     inherited Create(AOwner);
     FNeedSave :=False;
     FEval     :=nil;
     Style     := csOwnerDrawFixed;
     Position  := 1;
end;


procedure TEvalHistoryCombo.SetFileName(Value:String);
begin
     FFileName := Value;
     LoadItems;
end;

procedure TEvalHistoryCombo.SetFileNameKey(Value:String);
begin
     FFileNameKey := Value;
     LoadItems;
end;

procedure TEvalHistoryCombo.LoadItems;
var L  :TStringList;
    i,j:Integer;
begin
     Items.Clear;

     if (FFileName='') then Exit;
     if FFileNameKey='' then FFileNameKey:='#1';
     if not FileExists(FFileName) then Exit;
     L:= TStringList.Create;

     try
       L.LoadFromFile(FFileName);
       for i := 0 to  L.Count-1 do begin
           j:=Pos('|',L.Strings[i]);
           if j>0 then
              if Copy(L.Strings[i],1,j-1)=FFileNameKey then
                 Items.Add(L.Strings[i]);
       end;
     finally
       L.Free;
     end;
end;


procedure TEvalHistoryCombo.SaveItems;
var L  :TStringList;
    i,j:Integer;
begin
     if (FFileName='') then Exit;
     if FFileNameKey='' then FFileNameKey:='#1';

     L:= TStringList.Create;
     try
       if (FFileName<>'') and FileExists(FFileName) then
          L.LoadFromFile(FFileName);

       for i:=L.Count-1 downto 0 do begin
           j:=Pos(HistorySeparator,L.Strings[i]);
           if j>0 then
              if Copy(L.Strings[i],1,j-1)=FFileNameKey then
                 L.Delete(i);
       end;

       for i:=0 to Items.Count-1 do
           L.Add(Items[i]);

       L.SaveToFile(FFileName);
       FNeedSave:=False;
     finally
       L.Free;
     end;
end;

procedure TEvalHistoryCombo.DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState);
begin
     Canvas.TextRect(Rect, Rect.Left, Rect.Top, GetTextByIndex(index,1) );
end;

procedure   TEvalHistoryCombo.AddWithQuestion(Expr:String);
var s:string;
begin
    if InputQuery(rsHistoryCaption,rsHistoryPrompt,s) then
       AddCond(Expr,s);
end;

procedure   TEvalHistoryCombo.AddCond(Expr:String; Str:String);
begin
     if Str='' then
        if Eval<>nil then Str:=Eval.Expression;

     if Str<>'' then begin
        Items.Add(FFileNameKey+HistorySeparator+Str+HistorySeparator+Expr);
        SaveItems;
     end;
     //FNeedSave:=True;
end;

function    TEvalHistoryCombo.Expression:String;
begin
     Result:=GetExpressionByIndex(ItemIndex);
end;

function    TEvalHistoryCombo.GetCaption:String;
begin
     Result:=GetCaptionByIndex(ItemIndex);
end;


function    TEvalHistoryCombo.GetExpressionByIndex(idx:Integer):String;
var i : Integer;
begin
     Result:='';
     if (idx>=0) and (idx<Items.Count) then begin
        Result:=Items[idx];
        i:=Pos(HistorySeparator,Result);
        if i<=0 then Exit;
        Result:=Copy(Result,i+1,Length(Result));
        i:=Pos(HistorySeparator,Result);
        if i>1 then Result:=Copy(Result,i+1,10000);
     end;
end;

function    TEvalHistoryCombo.GetCaptionByIndex(idx:Integer):String;
var i : Integer;
begin
     Result:='';
     if (idx>=0) and (idx<Items.Count) then begin
        Result:=Items[idx];
        i:=Pos(HistorySeparator,Result);
        if i<=0 then Exit;
        Result:=Copy(Result,i+1,Length(Result));
        i:=Pos(HistorySeparator,Result);
        if i>1 then Result:=Copy(Result,1,i-1);
     end;
end;

procedure TEvalHistoryCombo.Change;
begin
     if FEval<>nil then
        FEval.Expression:=Expression;
     inherited Change;
end;

procedure TEvalHistoryCombo.Notification(AComponent: TComponent;
     Operation: TOperation);
begin
     if (Operation=opRemove) and (AComponent=FEval) then Eval:=nil;
     inherited Notification(AComponent, Operation);
end;



constructor TEvalCombo.Create(AOwner:TComponent);
begin
     inherited Create(AOwner);
     FPosition   := 0;
     Style       := csDropDown;
{     OnDrawItem  := MyDrawItem;}
end;

{procedure TEvalCombo.MyDrawItem(Control: TWinControl; Index: Integer; Rect: TRect;
              State: TOwnerDrawState);
var  Te: array[0..255] of Char;
begin
  with Canvas do begin
    FillRect(Rect);
    StrPCopy(Te, GetTextByIndex(Index,FPosition));
    Rect.Left := Rect.Left + 2;
    DrawText(Canvas.Handle, Te, StrLen(Te), Rect, DT_SINGLELINE or
      DT_VCENTER or DT_NOPREFIX);
  end;
end;
}

procedure TEvalCombo.SetPosition(Value:Integer);
begin
     if FPosition<>Value then begin
        FPosition:=Value;
        Invalidate;
     end;
end;


function TEvalCombo.GetTextByIndex(_Index,_Pos:Integer):String;
var s   : String;
    i,j : Integer;
begin
    s:='';
    if (_Index>=0) and (_Index<Items.Count) then s:=Items[_Index];

    Result:='';
    if s='' then Exit;

    i:=_pos;
    while i>0 do begin
          j:=Pos(HistorySeparator,s);
          if j>0 then s:=Copy(s,j+1,Length(s)-j)
          else Exit;
          Dec(i);
    end;
    j:=Pos(HistorySeparator,s);
    if j>0 then Result:=Copy(s,1,j-1)
    else        Result:=s;
end;


procedure TEvalCombo.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  //with Params do
  //  Style := Style or CBS_OWNERDRAWFIXED;
end;

procedure   TEvalCombo.DropDown;
begin
    inherited; 
    if ItemIndex>=0 then
             Text:=GetTextByIndex(ItemIndex,0);
end;

end.
