unit EvalFn;

interface

uses DB;

const WordDelims=' ;,./-?!';


type dmyName=(dmyDayName,dmyDayNameLong,dmyMonthName,dmyMonthNameLong,
              dmyYear, dmyMonth, dmyDay, dmyCentury, dmyDaysInMonth,
              dmyWeek, dmyHour, dmyMinute, dmySecond );

function PosWithError(Substr,Str:String; Errors:Integer):Boolean;
function StringAnywhere(DS:TDataSet; Substr:String):Boolean;
function AnsiStringAnywhere(DS:TDataSet; Substr:String):Boolean;
function StringBefore(Str,Separator:String):String;
function StringAfter(Str,Separator:String):String;
function ReplaceString(What,Where,ReplStr:String):String;
function DelChars(Str:String; What:String):String;
function Center(Str:String; Width:Integer; Ch:Char):String;
function MatchStrings(source, pattern: String): Boolean;
function WordCount(const S:String):Integer;
function GetWord(const S:String; idx:Integer):String;
Function WordInText(const S,T:String):Boolean;

function CapitalizeFirstLetter(S:String):String;
Function Soundex(OriginalWord: string): string;
function SoundLike(S1,S2:String):Boolean;
function NumberToSpell(X:Integer):String;

function DaysBetween(Da1,Da2:TDateTime):Integer;
function GetDMY(Da:TDateTime; What:dmyName):Variant;

function Sign(X:Double):Integer;
function Faktorial(X:Integer):Extended;
function Permutation(N,K:Integer):Extended;

function GetFileSize(const FileName: string): Longint;

function DecToRoman( Decimal: LongInt ): String;

implementation
uses SysUtils;

const DaysPerMonth: array[1..12] of Integer = (31, 28, 31, 30, 31, 30, 31,
      31, 30, 31, 30, 31);

  NumbersSpell : Array [0..30] of ShortString=(
      '','one','two','three','four','five','six',
      'seven','eight','nine','ten',
      'eleven','twelve','thirteen','fourteen','fifteen',
      'sixteen','seventeen','eighteen','nineteen',
      'twenty','thirty','fourty','fifty','sixty','seventy',
      'eighty','ninty','hundred','thousand','million');


function PosWithError(Substr,Str:String; Errors:Integer):Boolean;
var i,j,m,n       : Integer;
    ZnP,ZnT,TC,PC : String;
    f,f2,f3       : Integer;
    r0,r1         : array [1..100] of byte;
    P,T           : String;
begin
     P :=UpperCase(SubStr);
     T :=UpperCase(Str);

     m:=Length(P);
     n:=Length(T);

     if (m=0) or (n=0) or (m-n>errors ) then begin
        Result:=False;
        Exit;
     end;

     for i:=1 to m do r0[i]:=i;
     j:=0;
     f:=0;

     repeat
           Inc(j);
           PC  := '';
           ZnT := T[j];
           TC  := TC+ZnT;
           for i:=1 to m do begin
               ZnP := P[i];
               PC  := PC+ZnP;
               if i>1 then f := r0[i-1]
               else        f := 0;

               if ZnP<>ZnT then Inc(f);
               if Pos(ZnP,TC)<=0 then begin
                  if i>1 then  f2:=r1[i-1]+1
                  else         f2:=1;
                  if f2<f then f:=f2;
               end;
               if Pos(ZnT,PC)<=0 then begin
                  if i>1 then  f3:=r0[i]+1
                  else         f3:=1;
                  if f3<f then f:=f3;
               end;
               r1[i]:=f;
           end;
           if f>Errors then
              for i:=1 to 100 do r0[i]:=r1[i];
     until ((f<=Errors) or (j=n));

     Result:=(f<=Errors);
end;



function DaysBetween(Da1,Da2:TDateTime):Integer;
begin
     Result:= abs(trunc(da2) - trunc(da1));
end;


// search for string in any string field of dataset
function StringAnywhere(DS:TDataSet; Substr:String):Boolean;
var i:Integer;
begin
    Result:=False;  if DS=nil  then Exit;
    if not DS.Active then EXit;

    for i:=0 to DS.FieldCount-1 do
        if Pos(substr,DS.Fields[i].AsString)>0 then begin
              Result := True;
              Exit;
        end;
end;

function AnsiStringAnywhere(DS:TDataSet; Substr:String):Boolean;
var i:Integer;
begin
    Result:=False;  if DS=nil  then Exit;
    if not DS.Active then EXit;

    for i:=0 to DS.FieldCount-1 do begin
           if AnsiPos(Substr,DS.Fields[i].AsString)>0 then begin
              Result := True;
              Exit;
           end;
    end;
end;

function StringBefore(Str,Separator:String):String;
var i:Integer;
begin
     i:=Pos(Separator,Str);
     if i<=0 then Result:=Str
     else         Result:=Copy(Str,1,i-1);
end;


function StringAfter(Str,Separator:String):String;
var i:Integer;
begin
     i:=Pos(Separator,Str);
     if i<=0 then Result:=''
     else         Result:=Copy(Str,i+1,Length(Str)-i);
end;

function ReplaceString(What,Where,ReplStr:String):String;
var i,j:Integer;
begin
    j:=Length(what);
    i:=Pos(What,Where);
    if i>0 then
          Result:=Copy(Where,1,i-1)
                 +ReplStr
                 +ReplaceString(What,Copy(Where,i+j,1000000),ReplStr)
    else Result:=Where;
end;

function DelChars(Str:String; What:String):String;
var i:Integer;
begin
     result:='';
     for i:=1 to Length(Str) do
         if Pos(Str[i],What)<=0 then Result:=Result+Str[i];
end;

function Center(Str:String; Width:Integer; Ch:Char):String;
var i:Integer;
begin
    i:=width-Length(Str);
    if i<=0 then Result:=Str
    else
         Result:=Concat(StringOfChar(Ch,i div 2),
                         Str,
                         StringOfChar(Ch, Width-(i div 2)-Length(str)));
end;


function GetDMY(Da:TDateTime; What:dmyName):Variant;
var d,m,y,DayCount,X : word;
    HO,MI,SE,DE:Word;
begin
     DecodeDate(da,Y,M,D);
     DecodeTime(da,HO,MI,SE,DE);
     case what of
          dmyDayName       : Result:=ShortDayNames[DayOfWeek(Da)];
          dmyDayNameLong   : Result:=LongDayNames[DayOfWeek(Da)];
          dmyMonthName     : Result:=ShortMonthNames[M];
          dmyMonthNameLong : Result:=LongMonthNames[M];
          dmyYear          : Result:=Y;
          dmyMonth         : Result:=M;
          dmyDay           : Result:=D;
          dmyCentury       : Result:=(Y div 100)+1;
          dmyWeek          : begin
                               DayCount := D;
                               For X := 1 to ( M - 1 ) do
                                   Inc(DayCount, DaysPerMonth[X]);
                               Result := ( ( DayCount div 7 ) + 1 );
                             end;
          dmyDaysInMonth   : begin
                                  Result := DaysPerMonth[M];
                                  if (M=2) and IsLeapYear(Y) then Inc(Result);
                             end;
          dmyHour          : Result := HO;
          dmyMinute        : Result := MI;
          dmySecond        : Result := SE;
     end;
end;

function  Sign(X:Double):Integer;
begin
          if X>0 then Result:= 1
     else if X<0 then Result:=-1
     else             Result:= 0;
end;

function Faktorial(X:Integer):Extended;
var i:Integer;
begin
     if X<=1 then Result:=1
     else begin
          Result:=1;
          for i:=2 to X do Result:=Result*i;
     end;
end;

function Permutation(N,K:Integer):Extended;
begin
     Result:=Faktorial(N)/(Faktorial(K)*Faktorial(N-K));
end;


function GetFileSize(const FileName: string): Longint;
var sr: TSearchRec;
begin
  if FindFirst(ExpandFileName(FileName), faAnyFile, Sr) = 0 then Result := Sr.Size
  else                                                           Result := -1;
  FindClose(sr);
end;

function MatchStrings(source, pattern: String): Boolean;
var pSource: Array [0..255] of Char;
    pPattern: Array [0..255] of Char;
    function MatchPattern(element, pattern: PChar): Boolean;
             function IsPatternWild(pattern: PChar): Boolean;
             begin
                  Result := StrScan(pattern,'*') <> nil;
                  if not Result then Result := StrScan(pattern,'?') <> nil;
             end;
    begin
         if 0 = StrComp(pattern,'*') then      Result := True
         else if (element^ = Chr(0)) and (pattern^ <> Chr(0)) then Result := False
              else if element^ = Chr(0) then                       Result := True
         else begin
              case pattern^ of
                   '*': if MatchPattern(element,PChar(pattern)) then Result := True
                        else             Result := MatchPattern(PChar(element),PChar(pattern));
                   '?': Result := MatchPattern(PChar(element),PChar(pattern));
                   else if element^ = pattern^ then Result := MatchPattern(PChar(element),PChar(pattern))
                        else                        Result := False;
              end;
         end;
    end;
begin
     StrPCopy(pSource,source);
     StrPCopy(pPattern,pattern);
     Result := MatchPattern(pSource,pPattern);
end;

function WordCount(const S: string): Integer;
var SLen, I: Cardinal;
begin
  Result := 0;
  I      := 1;
  SLen   := Length(S);
  while I <= SLen do begin
    while (I <= SLen) and (Pos(S[I],WordDelims)>0) do Inc(I);
    if I <= SLen then Inc(Result);
    while (I <= SLen) and (Pos(S[I],WordDelims)<=0) do Inc(I);
  end;
end;

function GetWord(const S: string; idx:Integer): String;
var SLen, I ,wc: Integer;
begin
  Result := '';
  wc     := 0;
  I      := 1;
  SLen   := Length(S);
  while I <= SLen do begin
    while (I <= SLen) and (Pos(S[I],WordDelims)>0) do Inc(I);
    if I <= SLen then Inc(wc);
    if wc=idx then begin
        while (I <= SLen) and (Pos(S[I],WordDelims)<=0) do begin
              Result:=Result+S[i];
              Inc(i);
        end;
        Exit;
    end;
    while (I <= SLen) and (Pos(S[I],WordDelims)<=0) do Inc(I);
  end;
end;

Function WordInText(const S,T:String):Boolean;
begin
     Result:=False;
     // dopracovat
end;


function CapitalizeFirstLetter(S:String):String;
const separators=' .';
var LastWasSeparator : BOOLEAN;
    ndx              : INTEGER;
    s1               : string;
begin
  LastWasSeparator := TRUE;
  ndx              := 1;
  s                := AnsiLowerCase(s);
  s1               := AnsiUpperCase(s);
  while (ndx<=Length(s)) do begin
    if LastWasSeparator    then s[ndx] := s1[ndx];
    LastWasSeparator := Pos(s[ndx], separators)>0;
    inc(ndx);
  end;
  Result := s;
end;

Function Soundex(OriginalWord: string): string;
var tmp1,tmp2 : string;
    Count: integer;
begin
     Tmp1 := '';
     Tmp2 := '';
     OriginalWord := Uppercase(OriginalWord);
     Appendstr(Tmp1, OriginalWord[1]); {Use the first letter of the word}
     for Count := 2 to length(OriginalWord) do
         {Assign a numeric value to each letter, except the first}
         case OriginalWord[Count] of
              'B','F','P','V'                 : Appendstr(tmp1, '1');
              'C','G','J','K','Q','S','X','Z' : Appendstr(tmp1, '2');
              'D','T'                         : Appendstr(tmp1, '3');
              'L'                             : Appendstr(tmp1, '4');
              'M','N'                         : Appendstr(tmp1, '5');
              'R'                             : Appendstr(tmp1, '6');
              {All other letters, punctuation and numbers are ignored}
         end;
     Appendstr(tmp2, OriginalWord[1]);
     {Go through the result removing any consecutive duplicate numeric values.}
     for Count:=2 to length(tmp1) do
          if Tmp1[Count-1]<>Tmp1[Count] then
              Appendstr(Tmp2,tmp1[Count]);
     Result:=tmp2; {This is the soundex value}
end;


function SoundLike(S1,S2:String):Boolean;
begin
     Result:=(Soundex(S1)=Soundex(s2));
end;

function NumberToSpell(X:Integer):String;
var c:Integer;
  function WordTo1000(Y:Integer; Dve:Boolean):String;
  begin
       case Y of
            0..19      : Result:= NumbersSpell[Y];
            20 .. 29   : Result:=NumbersSpell[20]+ NumbersSpell[Y-20];
            30 .. 39   : Result:=NumbersSpell[21]+ NumbersSpell[Y-30];
            40 .. 49   : Result:=NumbersSpell[22]+ NumbersSpell[Y-40];
            50 .. 59   : Result:=NumbersSpell[23]+ NumbersSpell[Y-50];
            60 .. 69   : Result:=NumbersSpell[24]+ NumbersSpell[Y-60];
            70 .. 79   : Result:=NumbersSpell[25]+ NumbersSpell[Y-70];
            80 .. 89   : Result:=NumbersSpell[26]+ NumbersSpell[Y-80];
            90 .. 99   : Result:=NumbersSpell[27]+ NumbersSpell[Y-90];
            100 .. 999 : begin
                              Result:= WordTo1000(Y div 100,True)+NumbersSpell[28];
                              if (Y mod 100)<>0 then
                                 Result:=Result+' and '+WordTo1000(Y mod 100,False);
                         end;
       end;
  end;
begin
     Result:='';
     if X>=1000000 then begin
        c      := X div 1000000;
        Result := WordTo1000(c,False)+NumbersSpell[30];
        X      := X mod 1000000;
        if X<>0 then Result:=Result+' and ';
     end;
     if X>=1000 then begin
        Result:=Result+WordTo1000(X div 1000,True)+NumbersSpell[29];
        X      := X mod 1000;
        if X<>0 then Result:=Result+' and ';
     end;

     Result:=Result+WordTo1000(X,False);
end;

function DecToRoman( Decimal: LongInt ): String;
const
  Romans:  Array[1..13] of String =
    ( 'I', 'IV', 'V', 'IX', 'X', 'XL', 'L', 'XC', 'C', 'CD', 'D', 'CM', 'M' );

  Arabics: Array[1..13] of Integer =
    ( 1, 4, 5, 9, 10, 40, 50, 90, 100, 400, 500, 900, 1000);

var i       : Integer;
    scratch : String;
begin
  scratch := '';
  for i := 13 downto 1 do
    while ( Decimal >= Arabics[i] ) do begin
      Decimal := Decimal - Arabics[i];
      scratch := scratch + Romans[i];
    end;
  Result := scratch;
end;



end.



// CapitalizeFirstLetter (popis v tips14.htm)
// SoundLike (popis v tips14.htm)
// CurrenctToString (popis v tips14.htm)


// CPUID
