unit evalist;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs,Eval,DB,DBTables;

type
  TEvalList = class(TComponent)
  private
    FList : TList;
    FDataSet : TDataSet;
    FEnableDataSet : Boolean;
    FForceString : Boolean;
    procedure SetDataSet(AValue : TDataSet);
    procedure SetEnableDataSet(AValue : Boolean);
    procedure SetForceString(AValue : Boolean);
    function GetExpressions(pos : integer) : string;
    procedure SetExpressions(pos : integer; exp : string);
    function GetEvals(pos : integer) : TEval;
    function GetFields(index : integer) : TField;
    function GetCount : integer;
  protected
    property List : TList read FList;
  public
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    function   Add(exp : string):Integer;
    procedure  Insert(pos : integer; exp : string);
    procedure  Delete(pos : integer);
    procedure  Clear;
    function   CreateCalcField(FieldName : string; Size : Word;  Expr:String  ) : TField;
    property Expressions[pos:integer] : string read GetExpressions
      write SetExpressions;
    property Evals[pos:integer]    : TEval read GetEvals; default;
    property Fields[Index:Integer] : TField Read GetFields;
    property Count : integer read GetCount;
  published
    property EnableDataSet : Boolean read FEnableDataSet
      write SetEnableDataSet default True;
    property ForceString : Boolean read FForceString write SetForceString
      default True;
    property DataSet : TDataSet read FDataSet write SetDataSet;
  end;

implementation

constructor TEvalList.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  FList := TList.Create;
  FEnableDataSet := True;
  FForceString := True;
end;

destructor TEvalList.Destroy;
begin
  if FList <> nil then
    Clear;
  FList.Free;
  inherited Destroy;
end;

procedure TEvalList.Clear;
var
  i : integer;
begin
  for i:=0 to FList.Count-1 do
    evals[i].Free;
  FList.Clear;
end;

function TEvalList.GetCount : integer;
begin
  Result := FList.Count;
end;

function TEvalList.GetEvals(pos : integer) : TEval;
begin
  Result := TEval( FList.Items[pos] );
end;

function TEvalList.GetExpressions(pos : integer) : string;
begin
  Result := evals[pos].Expression;
end;

procedure TEvalList.SetExpressions(pos : integer; exp : string);
begin
  evals[pos].Expression := exp;
end;

procedure TEvalList.SetDataSet(AValue : TDataSet);
var
  i : integer;
begin
  if AValue <> FDataSet then begin
    FDataSet := AValue;
    for i:=0 to FList.Count-1 do
      evals[i].DataSet := FDataSet;
  end;
end;

procedure TEvalList.SetEnableDataSet(AValue : Boolean);
var
  i : integer;
begin
  if AValue <> FEnableDataSet then begin
    FEnableDataSet := AValue;
    for i:=0 to FList.Count-1 do
      evals[i].EnableDataSet := FEnableDataSet;
  end;
end;

procedure TEvalList.SetForceString(AValue : Boolean);
var
  i : integer;
begin
  if AValue <> FForceString then begin
    FForceString := AValue;
    for i:=0 to FList.Count-1 do
      evals[i].ForceString := FForceString;
  end;
end;

function TEvalList.Add(exp : string):Integer;
var TheEval : TEval;
begin
  TheEval := TEval.Create(Self);
  TheEval.DataSet := FDataSet;
  TheEval.EnableDataSet := FEnableDataSet;
  TheEval.ForceString := FForceString;
  TheEval.Expression := exp;
  Result:=FList.Add(TheEval);
end;

function TEvalList.CreateCalcField(FieldName : string; Size : Word;  Expr:String  ) : TField;
var TheEval : TEval;
begin
  TheEval               := TEval.Create(Self);
  TheEval.DataSet       := FDataSet;
  TheEval.EnableDataSet := FEnableDataSet;
  TheEval.ForceString   := FForceString;
  try
     Result := TheEval.CreateCalcField(FieldName, Size, Expr );
  finally
  end;
  FList.Add(TheEval);
end;

procedure TEvalList.Insert(pos : integer; exp : string);
var TheEval : TEval;
begin
  TheEval := TEval.Create(Self);
  TheEval.DataSet := FDataSet;
  TheEval.EnableDataSet := FEnableDataSet;
  TheEval.ForceString := FForceString;
  TheEval.Expression := exp;
  FList.Insert(pos,TheEval);
end;

procedure TEvalList.Delete(pos : integer);
begin
  evals[pos].Free;
  FList.Delete(pos);
  FList.Pack;
end;



function TEvalList.GetFields(index : integer) : TField;
begin
     if (index>=0) and (index<Count) then Result:=EVals[Index].Field
     else                                 Result:=nil;
end;

end.
