unit EvalSimp;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,
  Grids,StdCtrls,Eval,Dialogs,Forms,ExtCtrls;

type


  TEvalSimpleDesigner = class(TStringGrid)
  private
       FEval             : TEval;
       FEvalCombo        : TComboBox;
       FLastCol,FLastRow : Integer;
       FBadRowColor      : TColor;
       FOnEvalChange     : TNotifyEvent;
       FExpression       : String;
       FSecondArgIsValue : Boolean;
       FPanel            : TPanel;
       procedure         SetBadRowColor(Value:TColor);
       procedure         SetEval(Value:TEval);
  protected
       function     DataSetActive:Boolean;
       function     SelectCell(ACol, ARow: Longint): Boolean; override;
       procedure    DrawCell(ACol, ARow: Longint; ARect: TRect;
                                   AState: TGridDrawState); override;
       //procedure    CreateParams(var Params: TCreateParams); override;
  public
       constructor  Create(AOwner:TComponent); override;
       procedure    Notification(AComponent: TComponent; Operation: TOperation); override;
       procedure    EvalComboChange(Sender:TObject);
       procedure    Loaded; override;
       function     IsRowOK(ARow:Integer):Boolean;
       function     GetRowExpression(ARow:Integer):String;
       function     CalcExpression:String;
       procedure    SetComboFields;
       procedure    SetComboOperators;
       procedure    SetComboAndOr;
       procedure    SetComboValues;
  published
    property Eval        : TEval        Read FEval        Write SetEval;
    property BadRowColor : TColor       Read FBadRowColor Write SetBadRowColor;
    property OnEvalChange    : TNotifyEvent Read FOnEvalChange    Write FOnEvalChange;
    property Expression  : String       Read FExpression;
    property SecondArgIsValue : Boolean Read  FSecondArgIsValue
                                        Write FSecondArgIsValue;
  end;


implementation
uses DB,EvalRes;

constructor TEvalSimpleDesigner.Create(AOwner:TComponent);
begin
     inherited Create(AOwner);
     ControlStyle:= ControlStyle + [csAcceptsControls];

     FEval      := nil;
     ColCount   := 4;
     RowCount   := 20;
     FixedCols  := 0;
     FixedRows  := 0;
     FBadRowColor := $00CFCFFF;
     FExpression  := '';
     FLastRow     := -1;
     FLastCol     := -1;

     Options := Options - [goRangeSelect,goRowSizing,goColSizing];

     ColWidths[0]       :=  30;
     ColWidths[1]       := 100;
     ColWidths[2]       :=  50;
     ColWidths[3]       := 100;
     Width              := 305;


     FPanel             := TPanel.Create(Self);
     FPanel.Parent      := Self;
     FPanel.BevelOuter  := bvNone;
     FEvalCombo         := TComboBox.Create(Self);
     FEvalCombo.Parent  := FPanel;

     FEvalCombo.Name    := 'SimpleEvalCombo';
     FEvalCombo.Ctl3D   := False;
     FEvalCombo.Width   := ColWidths[1];
     // FEvalCombo.Sorted  := True;
     FEvalCombo.OnChange := EvalComboChange;
     FSecondArgIsValue  := False;

     DefaultRowHeight   :=  FEvalCombo.Height;
     FPanel.Height      :=  FEvalCombo.Height;
end;

procedure  TEvalSimpleDesigner.SetEval(Value:TEval);
begin
     if Value<>FEval then begin
        FEval:=Value;
        SelectCell(Col,Row);
     end;
end;

procedure TEvalSimpleDesigner.SetBadRowColor(Value:TColor);
begin
     if Value<>FBadRowColor then begin
        FBadRowColor:=Value;
        Invalidate;
     end;
end;

procedure   TEvalSimpleDesigner.Notification(AComponent: TComponent; Operation: TOperation);
var i,j:Integer;
begin
     inherited Notification(AComponent, Operation);
     if (Operation=opRemove) and (AComponent=FEval) then begin
        Eval   := nil;
        for i:=0 to ColCount-1 do
            for j:=0 to RowCount-1 do
                Cells[i,j]:='';
     end;
end;

procedure TEvalSimpleDesigner.SetComboValues;
var F : TField;
    B : TBookMark;
begin
     FEvalCombo.Items.Clear;
     if DataSetActive then
          with FEval.DataSet,FEvalCombo.Items do begin
               F :=FindField(Cells[1,FLastRow]);
               if F<>nil then begin
                  Screen.Cursor := crSqlWait;
                  B:=GetBookMark;
                  DisableControls;
                  First;
                  while not EOF do begin
                        if IndexOf(F.AsString)<0 then
                           Add(F.AsString);
                        Next;
                  end;
                  GotoBookmark(B);
                  FreeBookmark(B);
                  EnableControls;
                  Screen.Cursor := crDefault;
               end;
          end;
end;


procedure TEvalSimpleDesigner.SetComboFields;
var i:Integer;
begin
       FEvalCombo.Items.Clear;
       if DataSetActive then
          with FEval.DataSet do begin
            for i:=0 to FieldCount-1 do
                if Fields[i].DataType in [ftString,ftSmallInt,ftInteger,ftWord,
                ftCurrency,ftFloat,ftBoolean,ftDate,ftTime,ftDateTime,
                ftAutoInc] then
                FEvalCombo.Items.AddObject(Fields[i].FieldName,Pointer(i));
          end;
end;

procedure TEvalSimpleDesigner.SetComboOperators;
var F:TField;
begin
    if not DataSetActive then Exit;
    with FEvalCombo.Items do begin
       Clear;
       F:=FEval.DataSet.FindField(Cells[1,FLastRow]);
       if F=nil then Exit;

       case F.DataType of
          ftString : begin
                       Add('='); Add('=*'); Add('?');
                     end;
          ftSmallInt,ftInteger,
          ftWord,ftAutoInc,
          ftCurrency,ftFloat : begin
                       Add('='); Add('>'); Add('>=');
                       Add('<'); Add('<='); Add('<>');
                     end;
          ftBoolean : begin
                       Add(rsTrue);
                       Add(rsFalse);
                     end;
          ftDate,ftTime,ftDateTime : begin
                       Add('='); Add('>'); Add('>=');
                       Add('<'); Add('<='); Add('<>');
                     end;
       end;
    end;
end;

procedure TEvalSimpleDesigner.SetComboAndOr;
begin
  with FEvalCombo.Items do begin
     Clear;
     if (FLastCol<>0) or (FLastRow<>0) then begin
        Add(rsOr);
        Add(rsAnd);
        // Add(rsNotOr);
        // Add(rsNotAnd);
     end else begin
        Add('');
        Add(rsNot);
     end;
  end;
end;

function TEvalSimpleDesigner.DataSetActive:Boolean;
begin
     Result:=False;
     if FEval<>nil then
       if FEval.DataSet<>nil then
         Result:=True;
end;

function TEvalSimpleDesigner.SelectCell(ACol, ARow: Longint): Boolean;
var R:TRect;
    s:String;
begin
     Result := inherited SelectCell(ACol,ARow);

     FLastCol := ACol;
     FLastRow := ARow;

     FPanel.Visible:=False;

     R:=CellRect(ACol,ARow);
     FPanel.SetBounds(R.Left,R.Top,R.Right-R.Left+2,R.Bottom-R.Top+2);
     FEvalCombo.SetBounds(0,0,FPanel.Width,FPanel.Height);
     FEvalCombo.Height:=200;

     // FEvalCombo.Style     := csDropDown;
     case ACol of
          0   : SetComboAndOr;
          1   : SetComboFields;
          2   : SetComboOperators;
          3   : if FSecondArgIsValue then SetComboValues
                else                      SetComboFields;
     end;

     s:=Cells[ACol,ARow];
     if FEvalCombo.Style=csDropDown then
        FEvalCombo.Text := s
     else
        FEvalCombo.ItemIndex:= FEvalCombo.Items.IndexOf(s);

     FPanel.Visible := True;
     //FEvalCombo.SelectAll;
end;

procedure  TEvalSimpleDesigner.EvalComboChange(Sender:TObject);
begin
  try
     if FLastRow<>-1 then begin
        Cells[FLastCol,FLastRow]:=FEvalCombo.Text;
        if ((Cells[1,FLastRow]<>'') or (Cells[3,FLastRow]<>''))
           and (Cells[2,FLastRow]='') then
               Cells[2,FLastRow]:='=';
        InvalidateRow(FLastRow);
     end;
     FExpression := CalcExpression;

     if Assigned(FOnEvalChange) then FOnEvalChange(Self);
  except
        ShowMessage(rsErrorInCondition);
  end;
end;

procedure    TEvalSimpleDesigner.Loaded;
begin
     inherited Loaded;
     SelectCell(0,0);
     FPanel.Visible := False;
     FPanel.Color   := Color;
end;

function TEvalSimpleDesigner.IsRowOK(ARow:Integer):Boolean;
var i,j:Integer;
begin
     j:=0;
     for i:=0 to ColCount-1 do
         if (Cells[i,ARow]='') and ((i<>0) or (ARow<>0)) then Inc(j);
     Result:=(j=0) or (j=ColCount);
     if (Cells[0,ARow]<>'') and (Cells[2,ARow]<>'') and (Cells[0,ARow]=Cells[2,ARow]) then
        Result:=False;
     if (ARow>0) and (j=ColCount-1) and (Cells[0,ARow]='') then
        Cells[0,ARow]:='and';
end;

procedure TEvalSimpleDesigner.DrawCell(ACol, ARow: Longint; ARect: TRect;
            AState: TGridDrawState);
begin
     if not IsRowOk(ARow) then begin
        Canvas.Brush.Color:=FBadRowColor;
        Canvas.FillRect(ARect);
     end;

     inherited DrawCell(ACol,ARow,ARect,AState);
end;


function TEvalSimpleDesigner.GetRowExpression(ARow:Integer):String;
var s       : String;
    Var1,Var2,Op:String;
    F1,F2   : TField;
begin
     Result:='';
     if IsRowOK(ARow) and DataSetActive then begin
        var1:=Cells[1,ARow];
        op  :=Cells[2,ARow];
        var2:=Cells[3,ARow];
        F1:=FEval.DataSet.FindField(Var1);
        F2:=FEval.DataSet.FindField(Var2);
        if F1<> nil then
            case F1.DataType of
                       ftString,ftBCD : begin
                           if F2=nil then var2:=''''+Var2+'''';
                                if op='?'  then s:='Pos('+var2+','+var1+')>0'
                           else if op='='  then s:=var1+'='+var2
                           else if op='=*' then s:='Copy('+var1+',1,Length('+var2+'))='+var2;
                       end;
                       ftSmallint,
                       ftInteger,
                       ftWord,
                       ftBoolean,
                       ftFloat,
                       ftCurrency : s:=var1+op+var2;
                       ftDate     ,
                       ftTime     ,
                       ftDateTime : begin
                                  if F1=nil then s:='{'+var1+'}'
                                  else           s:=var1;
                                  s:=s+op;
                                  if F2=nil then s:=s+'{'+var2+'}'
                                  else           s:=s+var2;
                           end;
           end;
     end;
     Result:=s;
end;


function TEvalSimpleDesigner.CalcExpression:String;
var i:Integer;
    p,s:String;

begin
    Result:='';
    for i:=FixedRows to RowCount-1 do begin
        s:=GetRowExpression(i);
        if s<>'' then begin
           if Result<>'' then begin
              p:=Cells[0,i];
                   if (p=rsNotOr)  then Result := Result+' or (not('+s+'))'
              else if (p=rsNotAnd) then Result := Result+' and (not('+s+'))'
              else if (p=rsNot)    then Result := Result+' (not('+s+'))'
              else                      Result := Result+' '+p+'('+s+')';
           end else
              Result:=Result+'('+s+')';
        end;
    end;
end;

end.
