unit EvalExPa;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls,StdCtrls, EValDefs,EvalCom, EvalLbl;

type

    TEvalLabelType=(ltGroup,ltFunction,ltHint,ltExample,
        ltFunctionList,ltGroupList);

    TEvalFnPanel = class(TPanel)
    private
      FunctionGroup      : TEvalFnGroupList;
      FunctionList       : TEvalFnListBox;
      FLabelHi,FLabelFn,FLabelFg,FLabelEx    : TEvalLabel;
      FOnFunctionGroupChange              : TNotifyEvent;
      FOnFunctionChange                   : TNotifyEvent;
      FDistance                           : Integer;
      procedure FGChange(Sender: TObject);
      procedure FunctionListChange(Sender: TObject);
      procedure CreateControls;
      procedure SetFunctionType(Value:TFnType);
      function  GetFunctionType:TFnType;
      procedure SetSelectedFunction(Value:TEvalSymbol);
      function  GetSelectedFunction:TEvalSymbol;
      procedure FontChange(Sender: TObject);
      procedure SetDistance(Value:Integer);
  protected
      procedure   CreateWnd;                       override;
      procedure   RekalkExpr;
  public
      constructor Create(Aowner:TComponent);       override;
      destructor  Destroy;                         override;
      {function  GetChildParent: TComponent; override;
      function  GetChildOwner: TComponent;  override;
      procedure GetChildren(Proc: TGetChildProc; Root: TComponent); override;}
      procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
      procedure ArrangeControls;
      function  GetSubComponent(t:TEvalLabelType):TComponent;
  published
      property Distance      :Integer    Read FDistance Write SetDistance;
      // property LabelGroups   :TEvalLabel Read FLabelFg;
      // property LabelFunctions:TEvalLabel Read FLabelFn;
      // property LabelExample  :TEvalLabel Read FLabelEx;
      // property LabelHint     :TEvalLabel Read FLabelHi;
      property FunctionType:TFnType Read GetFunctionType Write SetFunctionType;
      property SelectedFunction:TEvalSymbol Read  GetSelectedFunction
                                            Write SetSelectedFunction;

      property OnFunctionGroupChange:TNotifyEvent Read  FOnFunctionGroupChange
                                                  Write FOnFunctionGroupChange;
      property OnFunctionChange     :TNotifyEvent Read  FOnFunctionChange
                                                  Write FOnFunctionChange;
    end;


implementation
uses EvalRes,Eval;

constructor TEvalFnPanel.Create(AOwner:TComponent);
begin
     inherited Create(AOwner);
     Width           := 315;
     Height          := 225;
     Caption         := ' ';
     FDistance       := 3;

     FunctionList  := TEvalFnListBox.Create(self);
     FunctionList.Parent  := Self;

     FunctionGroup := TEvalFnGroupList.Create(self);
     FunctionGroup.Parent := Self;


     FLabelHi         := TEvalLabel.Create(self);
     FLabelHi.Parent  :=Self;
     FLabelHi.Caption :='Hint';
     FLabelHi.AutoSize:=True;
     FLabelHi.Align   :=alBottom;

     FLabelEx         := TEvalLabel.Create(self);
     FLabelEx.Parent  :=Self;
     FLabelEx.Caption :='Example';
     FLabelEx.AutoSize:=True;
     FLabelEx.Align   :=alBottom;

     FLabelFg         := TEvalLabel.Create(self);
     FLabelFg.Parent  := Self;
     FLabelFg.Caption := rsFunctionGroupLabel;
     FLabelFg.AutoSize:=False;

     FLabelFn         := TEvalLabel.Create(self);
     FLabelFn.Parent  := Self;
     FLabelFn.Caption := rsFunctionLabel;
     FLabelFn.AutoSize:=False;

end;

destructor TEvalFnPanel.Destroy;
begin
      FunctionGroup.Free;
      FunctionList.Free;
      FLabelHi.Free;
      FLabelFn.Free;
      FLabelFg.Free;
      FLabelEx.Free;

      inherited Destroy;
end;


procedure TEvalFnPanel.CreateWnd;
begin
     inherited CreateWnd;
     CreateControls;
end;

procedure TEvalFnPanel.CreateControls;
begin
     // FunctionList       := TEvalFnListBox.Create(Self);
     with FunctionList do begin
          Parent   := Self;
          OnClick  := FunctionListChange;
          Font.OnChange := FontChange;
     end;

     // FunctionGroup := TEvalFnGroupList.Create(Self);
     with FunctionGroup do begin
          Parent   := Self;
          OnClick  := FGChange;
          Font.OnChange := FontChange;
     end;

     // FLabelEx:=TEvalLabel.Create(Self);
     FLabelEx.Parent:=Self;
     FLabelEx.Font.OnChange := FontChange;

     with FLabelHi do begin
          Parent:=Self;
          Font.OnChange := FontChange;
          Caption       := '';
          WordWrap      := True;
     end;

     // labels
     FLabelFg.Parent:=Self;
     FLabelFg.Font.OnChange := FontChange;
     FLabelFg.Caption := rsFunctionGroupLabel;

     // FLabelFn:=TEvalLabel.Create(Self);
     FLabelFn.Parent:=Self;
     FLabelFn.Caption := rsFunctionLabel;
     FLabelFg.Font.OnChange := FontChange;

     FunctionType := fnString;

     ArrangeControls;
end;

procedure TEvalFnPanel.FGChange(Sender: TObject);
begin
    if FunctionGroup<>nil then begin
       FunctionList.FunctionType:=FunctionGroup.FunctionType;
       if Assigned(FOnFunctionGroupChange) then
          FOnFunctionGroupChange(Self);
    end;
end;

procedure TEvalFnPanel.SetFunctionType(Value:TFnType);
begin
   if FunctionGroup<>nil then
      if Value<>FunctionGroup.FunctionType then begin
         FunctionGroup.FunctionType:=Value;
         FunctionList .FunctionType:=Value;
      end;
end;

function  TEvalFnPanel.GetFunctionType:TFnType;
begin
     Result:=FunctionGroup.FunctionType;
end;

procedure TEvalFnPanel.FunctionListChange(Sender: TObject);
var idx : Integer;
    F   : TDefFun;
begin
     if Assigned(FOnFunctionChange) then
        FOnFunctionChange(self);

     // set function hint
     if (FunctionList.Items.Count>FunctionList.ItemIndex) and (FunctionList.ItemIndex>=0) then
         idx:=Integer(FunctionList.Items.Objects[FunctionList.ItemIndex])
     else
         idx:=0;
     F:=GetFunctionByIndex(idx);
     FlabelHi.caption:=F.Hint;
     RekalkExpr;
     ArrangeControls;
end;

procedure   TEvalFnPanel.RekalkExpr;
var idx : Integer;
begin
     FLabelEx.Caption:='';
     if (FunctionList.Items.Count>FunctionList.ItemIndex) and (FunctionList.ItemIndex>=0) then begin
         idx:=Integer(FunctionList.Items.Objects[FunctionList.ItemIndex]);
         FLabelEx.Caption:=GetFnCall(idx);
     end;
end;

procedure TEvalFnPanel.SetSelectedFunction(Value:TEvalSymbol);
var F       : TDefFun;
begin
     if FunctionList  = nil then Exit;
     if FunctionGroup = nil then Exit;

     if SelectedFunction<>Value then begin
        F:=GetFunctionBySymbol(Value);
        FunctionGroup.FunctionType     := F.Kind;
        FunctionList.SelectedFunction  := F.FT;
        FunctionList.OnClick(Self);
     end;
end;

function  TEvalFnPanel.GetSelectedFunction:TEvalSymbol;
begin
     result:=esNone;
     if FunctionList=nil then Exit;
     Result:=FunctionList.SelectedFunction;
end;


procedure TEvalFnPanel.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
     inherited SetBounds(ALeft,ATop,AWidth,AHeight);
     ArrangeControls;
end;

procedure TEvalFnPanel.ArrangeControls;
var dy:Integer;
    wi: Integer;
    he : Integer;
begin
     if (FunctionGroup<>nil) and (FLabelHi<>nil) then begin
        wi:=(width-3*FDistance) div 2;
        dy:=FDistance;
        if FLabelFg.Visible or FLabelFn.Visible then begin
           FLabelFg.SetBounds(FDistance,     dy,  wi,FLabelFg.Height);
           FLabelFn.SetBounds(2*FDistance+wi,dy,wi,FLabelFn.Height);
           Inc(dy,FLabelFn.Height+FDistance);
        end;

        he:=Height-dy-FDistance;
        if FLabelEx.Visible then Dec(he,FLabelEx.Height);
        if FLabelHi.Visible then Dec(he,FLabelHi.Height);

        FunctionGroup.SetBounds(FDistance,     dy,wi,     he);
        FunctionList.SetBounds (FDistance+wi+1,dy,wi,     he);

     end;
end;

procedure TEvalFnPanel.SetDistance(Value:Integer);
begin
     if Value<>FDistance then begin
        FDistance:=Value;
        ArrangeControls;
     end;
end;

procedure TEvalFnPanel.FontChange(Sender: TObject);
begin
     ArrangeControls;
     Invalidate;
end;

{function TEvalFnPanel.GetChildParent: TComponent;
begin
     Result:=Self;
end;

function TEvalFnPanel.GetChildOwner: TComponent;
begin
     Result:=Self;
end;

procedure TEvalFnPanel.GetChildren(Proc: TGetChildProc; Root: TComponent);
begin
     inherited GetChildren(Proc,Root);
     Proc(FLabelHi);
     Proc(FLabelFn);
     Proc(FLabelFg);
     Proc(FLabelEx);
     Proc(FunctionList);
     Proc(FunctionGroup);
end;
}

function TEvalFnPanel.GetSubComponent(t:TEvalLabelType):TComponent;
begin
     case t of
          ltGroup    : Result:=TComponent(FLabelFg);
          ltFunction : Result:=TComponent(FLabelFn);
          ltHint     : Result:=TComponent(FLabelHi);
          ltExample  : Result:=TComponent(FLabelEx);
          ltFunctionList : Result:=TComponent(FunctionGroup);
          ltGroupList    : Result:=TComponent(FunctionList);
          else         Result:=nil;
     end;
end;

end.
